@@ -16,26 +16,32 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
+ *   51 Franklin Steet, Fifth Floor, Boston, MA  02110-1301, USA.          *
  ***************************************************************************/
 
-#include "last.fm.h"
 #include "container.h"
-#include "containerutils.h"
 #include "Scrobbler-1.2.h"
+#include "logger.h"
+
+#include <QApplication>
 #include <QDateTime>
+#include <QDir>
 #include <QDomDocument>
 #include <QDomElement>
 #include <QFile>
+#include <QTimer>
 
 
-//FIXME breaks encapsulation!
-#include "Settings.h"
-#include "MediaDeviceConfirmDialog.h"
+/** used to determine save location for scrobble caches 
+  * NOTE you'll need to define this somewhere */
+QString savePath( QString );
+
+/** returns the 32 character HEX representation of the supplied data
+  * NOTE you'll need to define this somewhere */
+QString MD5Digest( const char* );
 
 
-
-/** used to sort tracks into chronological order pre scrobbling submission*/
+/** used to sort tracks into chronological order pre scrobbling submission */
 static bool trackInfoLessThan( const TrackInfo &t1, const TrackInfo &t2)
 {
     return t1.timeStamp() < t2.timeStamp();
@@ -58,7 +64,7 @@ ScrobblerManager::~ScrobblerManager()
             qDebug() << "Deleting expired backup:" << path;
             QFile::remove( path );
         }
-    
+
 //////
     qDeleteAll( m_scrobblers );
 }
@@ -70,61 +76,39 @@ ScrobblerManager::handshake( const Scrobbler::Init& init )
     Q_DEBUG_BLOCK << init.username;
 
     Scrobbler* scrobbler = scrobblerForUser( init.username );
-    
-    //TODO should prolly delete the other and make a new one, although wait for other to finish any submission first
+
     if (!scrobbler) {
         scrobbler = new Scrobbler( init );
-        
+
         connect( scrobbler, SIGNAL(handshaken( Scrobbler* )), SLOT(onHandshaken( Scrobbler* )) );
         connect( scrobbler, SIGNAL(scrobbled( QList<TrackInfo> )), SLOT(onScrobbled( QList<TrackInfo> )) );
         connect( scrobbler, SIGNAL(invalidated( int )), SLOT(onInvalidated( int )) );
-    
+
         m_scrobblers += scrobbler;
-        
+
         emit status( Scrobbler::Connecting );
     }
 }
-    
+
 
 void //public
-ScrobblerManager::scrobble( const TrackInfo& track )
+ScrobblerManager::scrobble( TrackInfo track )
 {
     Q_DEBUG_BLOCK << track.toString();
     Q_ASSERT( !track.isEmpty() );
 
-    //HACK skip hack, see Container::skip()
-    if (track.isSkipped())
-    {
-        ScrobbleCache cache( track.username() );
-        cache.append( track );
-        
-        // we return as the Player* classes may also submit this track
-        // and we need to wait and see if they do and not scrobble both
-        // just this one
-        return;
-    }
-    //HACK
-    
-    if (track.playerId() == "itw" || track.playerId() == "osx")
-    {
-        ItunesScrobbleHistory().append( track );
-    }
-
     ScrobbleCache cache( track.username() );
-    
-    //HACK skip hack, see Container::skip()
-    foreach (TrackInfo cachedtrack, cache.tracks())
-        if (cachedtrack.isSkipped() && track.sameAs( cachedtrack ) && track.timeStamp() == cachedtrack.timeStamp())
-        {
-            qDebug() << "Ignoring the skip, scrobble point was reached";
-            
-            cache.remove( QList<TrackInfo>() << cachedtrack );
-            break;
-        }
-    //HACK
-    
     cache.append( track );
-    scrobble( cache );
+
+    //HACK skip hack, see Container::skip()
+    //NOTE this won't break scrobbling if the listener ever includes the rating
+    // in its trackinfo object but it will delay that scrobble until a track
+    // with no rating is submitted
+    // we don't scrobble as the Player* classes may also submit this track
+    // and we need to wait and see if they do and not scrobble both
+    // just this one
+    if ( !track.isSkippedLovedOrBanned() )
+        scrobble( cache );
 }
 
 
@@ -133,64 +117,36 @@ ScrobblerManager::scrobble( const ScrobbleCache& cache )
 {
     Q_DEBUG_BLOCK << cache.username();
     Q_ASSERT( cache.username().length() );
-    
+
     if (cache.tracks().isEmpty())
         qDebug() << "No tracks to scrobble";
-    
-    else if (Container::instance().isClosingDown())
+
+    else if (qApp->closingDown())
         // if we are shutting down, save the submit for next time
         // we choose this method as otherwise we have to hang the app waiting for a 
         // timeout and this isn't safe --mxcl
         qDebug() << "Not actually submitting as we are shutting down";
-    
+
     else if (Scrobbler* scrobbler = scrobblerForUser( cache.username() ))
     {
         int const N = scrobbler->submit( cache );
-    
+
         if (N)
             emit status( Scrobbler::Scrobbling, N );
         else
             qDebug() << "Scrobbler not ready, will submit at earliest opportunity";
     }
     else
-        qWarning() << "No scrobbler available for:" << cache.username();
-}
-
-
-void
-ScrobblerManager::submitMediaDeviceCache( const QString& username )
-{
-    Scrobbler* scrobbler = scrobblerForUser( username );
-    
-    if (scrobbler)
-    {
-        LOGL( 3, "Found scrobbler for user " << username << ", submitting..." );
-
-        ScrobbleCache cache( username );
-        cache.transferMediaDeviceCache();
-        scrobble( cache );
-    }
-    else
-    {
-        LOGL( 3, "Found no scrobbler for user " << username << ", creating a new one..." );
-
-        Scrobbler::Init init;
-        init.username = username;
-        init.password = The::user( username ).password();
-        init.client_version = The::settings().version();
-        
-        handshake( init );
-    }
+        qDebug() << "No scrobbler available for:" << cache.username();
 }
 
 
 void
 ScrobblerManager::nowPlaying( const TrackInfo& track )
 {
-    Scrobbler* scrobbler = scrobblerForUser( track.username() );
-
     Q_DEBUG_BLOCK << track.toString();
-    Q_ASSERT( scrobbler );
+
+    Scrobbler* scrobbler = scrobblerForUser( track.username() );
 
     if (scrobbler && scrobbler->canAnnounce())
         //TODO cache the now playing if the scrobbler is not yet ready
@@ -206,7 +162,7 @@ ScrobblerManager::scrobblerForUser( const QString& username ) const
     foreach (Scrobbler* s, m_scrobblers)
         if (s->username() == username)
             return s;
-    
+
     return 0;
 }
 
@@ -219,7 +175,7 @@ ScrobblerManager::onInvalidated( int code )
     Scrobbler* scrobbler = (Scrobbler*)sender();
     scrobbler->deleteLater();
     int const N = m_scrobblers.removeAll( scrobbler );
-    
+
     Q_ASSERT( N == 1 );
 
     switch (code)
@@ -248,19 +204,25 @@ ScrobblerManager::onScrobbled( const QList<TrackInfo>& tracks )
     Q_ASSERT( sender() );
 
 //////
-    ScrobbleCache cache( static_cast<Scrobbler*>(sender())->username() );
-    
+    Scrobbler* scrobbler = static_cast<Scrobbler*>(sender());
+    ScrobbleCache cache( scrobbler->username() );
+
     if (tracks.count() > 2)
         //do a backup because nobody writes perfect code, least of all me!
         cache.backup();
 
     int remaining = cache.remove( tracks );
-    
+
     if (remaining)
         scrobble( cache );
-    else
-        // the above scrobble() will emit a status message too, so only do one or the other
-        emit status( Scrobbler::TracksScrobbled, tracks.count() );
+    else {
+        // only show status on final submission batch, and only if something
+        // was scrobbled (not skipped or banned)
+        if (scrobbler->scrobbled() > 0)
+            emit status( Scrobbler::TracksScrobbled, scrobbler->scrobbled() );
+
+        scrobbler->resetScrobbleCount();
+    }
 }
 
 
@@ -268,15 +230,14 @@ void
 ScrobblerManager::onHandshaken( Scrobbler* scrobbler )
 {
     QString const username = scrobbler->username();
-    ScrobbleCache cache( username );
-    
+
     Q_DEBUG_BLOCK << username;
     Q_ASSERT( username.length() );
 
-    emit status( Scrobbler::Handshaken );
-
-    cache.transferMediaDeviceCache();
+    ScrobbleCache cache( username );
     scrobble( cache );
+
+    emit status( Scrobbler::Handshaken, username );
 }
 ///////////////////////////////////////////////////////////////////////////////>
 
@@ -286,9 +247,9 @@ ScrobbleCache::ScrobbleCache( const QString& username )
 {
     Q_ASSERT( username.length() );
 
-    m_path = savePath( username + "_submissions.xml" );
+    m_path = MooseUtils::savePath( username + "_submissions.xml" );
     m_username = username;
-    
+
     read();
 }
 
@@ -321,15 +282,15 @@ ScrobbleCache::mediaDeviceCache( const QString& username )
 }
 
 
-ScrobbleCache //static
+ScrobbleCache //static private
 ScrobbleCache::fromFile( const QString& filename )
 {
     ScrobbleCache cache;
-    cache.m_path = savePath( filename );
+    cache.m_path = MooseUtils::savePath( filename );
     cache.read();
-    
+
     //NOTE we don't set username, that's up to the caller
-    
+
     return cache;
 }
 
@@ -338,7 +299,7 @@ void
 ScrobbleCache::backup()
 {
     QString timestamp = QDateTime::currentDateTime().toString( "yyyyMMddhhmm" );
-    QString filename = QFileInfo( m_path ).baseName() + '.' + timestamp + ".backup.xml";
+        QString filename = QFileInfo( m_path ).baseName() + '.' + timestamp + ".backup.xml";
     ScrobbleCache backup = ScrobbleCache::fromFile( filename );
 
     // append in case we made a backup this minute already
@@ -351,7 +312,7 @@ ScrobbleCache::pathsForCacheBackups()
 {
     QStringList paths;
 
-    QDir d = savePath( "" );
+    QDir d = MooseUtils::savePath( "" );
     foreach (QString path, d.entryList( QStringList("*_submissions.*.backup.xml"), QDir::Files ))
         paths += d.filePath( path );
 
@@ -365,7 +326,6 @@ ScrobbleCache::write()
     if (m_tracks.isEmpty())
     {
         QFile::remove( m_path );
-    
         qDebug() << m_path << "is now empty";
     }
     else {
@@ -373,20 +333,20 @@ ScrobbleCache::write()
         QDomElement e = xml.createElement( "submissions" );
         e.setAttribute( "product", "Audioscrobbler" );
         e.setAttribute( "version", "1.2" );
-    
+
         foreach (TrackInfo i, m_tracks)
             e.appendChild( i.toDomElement( xml ) );
-    
+
         xml.appendChild( e );
-    
+
         QFile file( m_path );
         file.open( QIODevice::WriteOnly | QIODevice::Text );
-        
+
         QTextStream stream( &file );
         stream.setCodec( "UTF-8" );
         stream << "<?xml version='1.0' encoding='utf-8'?>\n";
         stream << xml.toString( 2 );
-    
+
         qDebug() << "Wrote" << m_tracks.count() << "tracks to" << m_path;
     }
 }
@@ -402,11 +362,47 @@ ScrobbleCache::append( const TrackInfo& track )
 void
 ScrobbleCache::append( const QList<TrackInfo>& tracks )
 {
-    m_tracks += tracks;
+    foreach (const TrackInfo& track, tracks)
+        merge( track );
     write();
 }
 
 
+//HACK
+void
+ScrobbleCache::merge( const TrackInfo& track )
+{
+    // we can't scrobble empties
+    if (track.isEmpty()) {
+        LOGL( 3, "Will not cache an empty track" );
+        return;
+    }
+
+    if (QDateTime::fromTime_t(track.timeStamp()) < QDateTime::fromString( "2003-01-01", Qt::ISODate ))
+    {
+        LOGL( 3, "Won't scrobble track from before the date Audioscrobbler project was founded!" );
+        return;
+    }
+
+    QMutableListIterator<TrackInfo> i( m_tracks );
+    while (i.hasNext())
+    {
+        TrackInfo& cachedtrack = i.next();
+
+        if (track.sameAs( cachedtrack ) && track.timeStamp() == cachedtrack.timeStamp())
+        {
+            // This will make sure no information gets lost
+            cachedtrack.merge( track );
+            return;
+        }
+    }
+
+    m_tracks += track;
+
+}
+//HACK
+
+
 int
 ScrobbleCache::remove( const QList<TrackInfo>& toremove )
 {
@@ -422,71 +418,22 @@ ScrobbleCache::remove( const QList<TrackInfo>& toremove )
                 i.remove();
             }
     }
-    
+
     qDebug() << m_tracks.count();
-    
+
     write();
-    
+
     // yes we return # remaining, rather # removed, but this is an internal 
     // function and the behaviour is documented so it's alright imo --mxcl
     return m_tracks.count();
 }
-
-
-void
-ScrobbleCache::transferMediaDeviceCache()
-{
-    //FIXME it breaks encapsulation to have this dialog here, and really, any
-    // kind of media device specific scrobble handling here
-    //TODO it's actually easy now to move this to where it is used and restore
-    // encapsulation
-
-    MediaDeviceConfirmDialog dialog( m_username, &Container::instance() );
-    
-    if (dialog.exec() == QDialog::Accepted)
-    {
-        //HACK note really we shouldn't do it this way, or at least reset the
-        // playcount to 1 for each, however we expect the protocol to be updated
-        // with playcount support soon.
-        // copy tracks playCount() times so we submit the correct number of plays
-        QList<TrackInfo> tracks;
-        foreach (TrackInfo const t, dialog.tracks())
-            for (int y = 0; y < t.playCount(); ++y)
-                tracks += t;
-        
-        append( tracks );
-    }
-    
-    QString const path = ScrobbleCache::mediaDeviceCache( m_username ).path();
-    
-    #ifndef LINUX
-//////HACK this behaviour doesn't reflect the function's name
-    QFile f( path );
-    f.open( QFile::Text | QFile::ReadOnly );
-    QDomDocument xml;
-    xml.setContent( &f );
-    QString const databaseLastModified = xml.documentElement().attribute( "lastItunesUpdate" );
-    f.close();
-    
-    // will be empty if not an iPod mediadevice cache
-    if (!databaseLastModified.isEmpty())
-    {
-        ItunesScrobbleHistory().prune( databaseLastModified.toUInt() );
-    }
-    #endif
-
-//////HACK this is proper behaviour, but doesn't reflect function name, instead 
-    // make a transferCache function delete the other cache so we don't have
-    // duplicate values in two caches
-    QFile::remove( path );
-}
 ///////////////////////////////////////////////////////////////////////////////>
 
 
 ///////////////////////////////////////////////////////////////////////////////>
 ItunesScrobbleHistory::ItunesScrobbleHistory()
 {
-    m_path = savePath( "iTunesScrobbleHistory.xml" );
+    m_path = MooseUtils::savePath( "iTunesScrobbleHistory.xml" );
     read();
 }
 
@@ -501,16 +448,16 @@ ItunesScrobbleHistory::append( const TrackInfo& candidate )
         if (track.sameAs( candidate )) 
         {
             int n = track.playCount() + 1;
-        
+
             track = candidate; //thus timestamp is set
             track.setPlayCount( n );
             write();
             return;
         }
     }
-    
+
     // if we get here then that track isn't in the itunes history file
-    
+
     ScrobbleCache::append( candidate );
 }
 
@@ -531,28 +478,29 @@ ItunesScrobbleHistory::prune( const time_t remove_if_before )
 ///////////////////////////////////////////////////////////////////////////////>
 Scrobbler::Scrobbler( const Scrobbler::Init& init )
 {
-    m_hard_failures = 0;
+    m_lastError = Scrobbler::ErrorNotInitialized;
+    m_hard_failures = m_scrobbled = 0;
     m_init = init;
 
     m_handshake = new ScrobblerHandshakeRequest( this );
-    m_now_playing = new ScrobblerPostRequest( this );
+    m_now_playing = new ScrobblerNowPlayingRequest( this );
     m_submission = new ScrobblerPostRequest( this );
-    
+
     // the QueuedConnection is required as Http causes a crash if you spawn a new
     // event loop in a slot connected to any of its dataAvailable type functions
     connect( m_handshake, SIGNAL(done( QString )), SLOT(onHandshakeReturn( QString )), Qt::QueuedConnection );
     connect( m_now_playing, SIGNAL(done( QString )), SLOT(onNowPlayingReturn( QString )), Qt::QueuedConnection );
     connect( m_submission, SIGNAL(done( QString )), SLOT(onSubmissionReturn( QString )), Qt::QueuedConnection );
-    
+
     connect( m_handshake, SIGNAL(responseHeaderReceived( QHttpResponseHeader )), SLOT(onHandshakeHeaderReceived( QHttpResponseHeader )) );
-    
+
     connect( this, SIGNAL(invalidated( int )), m_handshake, SLOT(abort()) );
     connect( this, SIGNAL(invalidated( int )), m_now_playing, SLOT(abort()) );
     connect( this, SIGNAL(invalidated( int )), m_submission, SLOT(abort()) );
-    
+
 //////
     qDebug() << "Initiating Scrobbler handshake for:" << m_init.username;
-    
+
     m_handshake->setHost( "post.audioscrobbler.com" );
     m_handshake->request( init );
 }
@@ -574,17 +522,18 @@ Scrobbler::hardFailure()
     ScrobblerHttp* http = (ScrobblerHttp*)sender();
     qDebug() << "Scrobbler HTTP error:" << http->id();
     http->abort();
-    
+
+    m_lastError = Scrobbler::ErrorNotInitialized;
     if (http != m_handshake && ++m_hard_failures >= 3)
     {
         qDebug() << "Three hard failures. Invalidating this Scrobbler.";
-    
+
         // ScrobblerManager will delete us and salvage submissions (if any)
         emit invalidated();
     }
     else {
         int const interval = http->retry();
-        
+
         qDebug() << "Scrobbler hard failure. Retrying in" << interval / 1000 << "seconds.";
     }
 }
@@ -601,30 +550,34 @@ Scrobbler::onHandshakeReturn( const QString& result )
 //////
     if (code == "OK" && results.count() >= 4)
     {
+        m_lastError = Scrobbler::NoError;
         m_session_id = results[1];
         m_now_playing->setUrl( results[2] );
         m_submission->setUrl( results[3] );
-        
+
         // reset hard failure state
         m_hard_failures = 0;
         m_handshake->resetRetryTimer();
-        
+
         emit handshaken( this );
-        
+
         // save memory *shrug*
         delete m_handshake;
         m_handshake = 0;
     }
     else if (code == "BANNED")
     {
+        m_lastError = Scrobbler::ErrorBannedClient;
         emit invalidated( Scrobbler::ErrorBannedClient );
     }
     else if (code == "BADAUTH")
     {
+        m_lastError = Scrobbler::ErrorBadAuthorisation;
         emit invalidated( Scrobbler::ErrorBadAuthorisation );
     }
     else if (code == "BADTIME")
     {
+        m_lastError = Scrobbler::ErrorBadTime;
         emit invalidated( Scrobbler::ErrorBadTime );
     }
     else
@@ -636,23 +589,20 @@ Scrobbler::onHandshakeReturn( const QString& result )
 int
 Scrobbler::submit( const ScrobbleCache& cache )
 {
-    if (!canSubmit())
+    if ( !canSubmit() )
         return 0;
 
     QList<TrackInfo> tracks = cache.tracks();
-    
+
     // we need to put the tracks in chronological order or the Scrobbling Service
     // rejects the ones that are later than previously submitted tracks
     // this is only relevent if the cache is greater than 50 in size as then
     // submissions are done in batches, but better safe than sorry
     qSort( tracks.begin(), tracks.end(), trackInfoLessThan );
     tracks = tracks.mid( 0, 50 );
-    
-    foreach (TrackInfo track, tracks)
-    	qDebug() << track.toString() << track.timeStamp();
-    
+
 //////
-    Q_DEBUG_BLOCK << tracks.count();
+    Q_DEBUG_BLOCK << (tracks.count() == 1 ? tracks[0].toString() : QString("%1 tracks").arg( tracks.count() ) );
     Q_ASSERT( m_submitted_tracks.isEmpty() );
     Q_ASSERT( m_session_id.size() );
     Q_ASSERT( tracks.size() <= 50 );
@@ -661,34 +611,33 @@ Scrobbler::submit( const ScrobbleCache& cache )
     QString data = "s=" + m_session_id;
     bool portable = false;
     int n = 0;
-    
+
     foreach (TrackInfo const i, tracks)
     {
         QString const N = QString::number( n++ );
-        
-        QString rating;
-        if (i.isSkipped()) rating += "S";
-        
         #define e( x ) QUrl::toPercentEncoding( x )
         data += "&a[" + N + "]=" + e(i.artist()) +
                 "&t[" + N + "]=" + e(i.track()) +
-                "&i[" + N + "]=" + e(QString::number( i.timeStamp() )) +
-                "&o[" + N + "]=" + e(i.sourceString() + i.authCode()) +
-                "&r[" + N + "]=" + rating +
+                "&i[" + N + "]=" + QString::number( i.timeStamp() ) +
+                "&o[" + N + "]=" + i.sourceString() + i.authCode() +
+                "&r[" + N + "]=" + i.ratingCharacter() +
                 "&l[" + N + "]=" + e(QString::number( i.duration() )) +
                 "&b[" + N + "]=" + e(i.album()) +
                 "&n[" + N + "]=" + //position in album if known, and we don't generally
-                "&m[" + N + "]=" + e(i.mbId());
+                "&m[" + N + "]=" + i.mbId();
         #undef e
-        
+
         if (i.source() == TrackInfo::MediaDevice)
             portable = true;
     }
-        
+
+    if (portable)
+        data += "&portable=1";
+
 //////
     m_submission->request( data.toUtf8() );
     m_submitted_tracks = tracks;
-    
+
     return tracks.count();
 }
 
@@ -699,24 +648,30 @@ Scrobbler::onSubmissionReturn( const QString& result )
     Q_DEBUG_BLOCK << "[id:" << ((ScrobblerHttp*)sender())->id() << "]" << result.trimmed();
     Q_ASSERT( m_submitted_tracks.count() );
 
-//////    
+//////
     QString const code = result.split( '\n' ).value( 0 );
-        
+
     if (code == "OK")
     {
+        m_lastError = Scrobbler::NoError;
+
+        foreach (const TrackInfo& track, m_submitted_tracks)
+            if (track.isLoved() || track.isScrobbled())
+                m_scrobbled++; //displayed to user after a complete cache is submitted
+
         m_hard_failures = 0;
         m_submission->resetRetryTimer();
-        
+
         // we must clear so that if more submissions are required they can proceed
         // as canSubmit() returns false if there are submitted tracks in the queue
         QList<TrackInfo> cp = m_submitted_tracks;
         m_submitted_tracks.clear();
-    
         emit scrobbled( cp );
     }
     else {
         if (code == "BADSESSION")
         {
+            m_lastError = Scrobbler::ErrorBadSession;
             emit invalidated( Scrobbler::ErrorBadSession );
         }
         else
@@ -731,6 +686,11 @@ Scrobbler::announce( const TrackInfo& track )
     Q_DEBUG_BLOCK << track.toString();
     Q_ASSERT( m_session_id.size() );
 
+    if (track.isEmpty()) {
+        LOGL( 3, "Empty track, not announcing to now playing" );
+        return;
+    }
+
 //////
     #define e( x ) QUrl::toPercentEncoding( x )
     QString data =  "s=" + e(m_session_id)
@@ -742,33 +702,67 @@ Scrobbler::announce( const TrackInfo& track )
                  + "&m=" + e(track.mbId());
     #undef e
 
-    m_now_playing->request( data.toUtf8() );
+    m_now_playing->ScrobblerPostRequest::request( data.toUtf8() );
 }
 
 
 void
 Scrobbler::onNowPlayingReturn( const QString& result )
 {
-    Q_DEBUG_BLOCK << "[id:" << ((ScrobblerHttp*)sender())->id() << "]" << result.trimmed();
+    qDebug() << "onNowPlayingReturn: [id:" << ((ScrobblerHttp*)sender())->id() << "]" << result.trimmed();
 
 //////
     QString const code = result.split( '\n' ).value( 0 );
-   
-    if (code == "OK") {
+
+    if (code == "OK")
+    {
+        m_lastError = Scrobbler::NoError;
         m_hard_failures = 0;
         m_now_playing->resetRetryTimer();
     }
     else if (code == "BADSESSION")
+    {
+        m_lastError = Scrobbler::ErrorBadSession;
         emit invalidated( Scrobbler::ErrorBadSession );
+    }
     else
         hardFailure();
 }
+
+
+QString
+Scrobbler::errorDescription( Scrobbler::Error error )
+{
+    switch ( error )
+    {
+        case Scrobbler::ErrorBadSession:
+            return tr( "Bad session" );
+
+        case Scrobbler::ErrorBannedClient:
+            return tr( "Client too old" );
+
+        case Scrobbler::ErrorBadAuthorisation:
+            return tr( "Wrong username / password" );
+
+        case Scrobbler::ErrorBadTime:
+            return tr( "Wrong timezone" );
+
+        case Scrobbler::ErrorNotInitialized:
+            return tr( "Could not reach server" );
+
+        default:
+            return "OK";
+    }
+}
+
+
+
 ///////////////////////////////////////////////////////////////////////////////>
 
 
 ///////////////////////////////////////////////////////////////////////////////>
 ScrobblerHttp::ScrobblerHttp( QObject* parent )
-            : Http( parent ),
+            : CachedHttp( parent ),
               m_id( 0 )
 {
     m_retry_timer = new QTimer( this );
@@ -790,13 +784,13 @@ ScrobblerHttp::onRequestFinished( int id, bool error )
     {
         if (error)
             qDebug() << this;
-        
+
         m_id = 0;
         emit done( error ? QString() : QString( readAll() ) );
     }
 }
 
-  
+
 void
 ScrobblerPostRequest::setUrl( const QUrl& url )
 {
@@ -804,7 +798,7 @@ ScrobblerPostRequest::setUrl( const QUrl& url )
     m_host = url.host();
     setHost( m_host, url.port() );
 }
-    
+
 
 int
 ScrobblerHttp::retry()
@@ -812,9 +806,9 @@ ScrobblerHttp::retry()
     int const i = m_retry_timer->interval();
     if (i < 120 * 60 * 1000)
         m_retry_timer->setInterval( i * 2 );
-    
+
     m_retry_timer->start();
-    
+
     return m_retry_timer->interval();
 }
 
@@ -832,7 +826,7 @@ void
 ScrobblerHandshakeRequest::request()
 {
     QString timestamp = QString::number( QDateTime::currentDateTime().toTime_t() );
-    QString auth_token = MD5Digest( (m_init.password + timestamp).toUtf8() );
+    QString auth_token = UnicornUtils::md5Digest( (m_init.password + timestamp).toUtf8() );
 
     QString query_string = QString() +
                             "?hs=true" +
@@ -844,7 +838,54 @@ ScrobblerHandshakeRequest::request()
                             "&a=" + auth_token;
 
     m_id = get( '/' + query_string );
-    
+
     qDebug() << "GET:" << query_string;
 }
 ///////////////////////////////////////////////////////////////////////////////>
+
+
+///////////////////////////////////////////////////////////////////////////////>
+void
+ScrobblerPostRequest::request()
+{
+    QHttpRequestHeader header( "POST", m_path );
+    header.setValue( "Host", m_host );
+    header.setContentType( "application/x-www-form-urlencoded" );
+
+    qDebug() << "POST:" << m_data;
+
+    m_id = CachedHttp::request( header, m_data );
+}
+
+
+void
+ScrobblerPostRequest::request( const QByteArray& data )
+{
+    Q_DEBUG_BLOCK << "[id:" << m_id << ']' << data;
+
+    m_data = data;
+    request();
+}
+///////////////////////////////////////////////////////////////////////////////>
+
+
+///////////////////////////////////////////////////////////////////////////////>
+ScrobblerNowPlayingRequest::ScrobblerNowPlayingRequest( QObject* parent )
+        : ScrobblerPostRequest( parent )
+{
+    m_timer = new QTimer( this );
+    m_timer->setInterval( 5000 );
+    m_timer->setSingleShot( true );
+    connect( m_timer, SIGNAL(timeout()), SLOT(request()) );
+}
+
+
+void
+ScrobblerNowPlayingRequest::request()
+{
+    if (sender())
+        ScrobblerPostRequest::request();
+    else
+        m_timer->start();
+}
+///////////////////////////////////////////////////////////////////////////////>
