@@ -2,6 +2,7 @@
  *   Copyright (C) 2005 - 2007 by                                          *
  *      Christian Muehlhaeuser, Last.fm Ltd <chris@last.fm>                *
  *      Erik Jaelevik, Last.fm Ltd <erik@last.fm>                          *
+ *      Jono Cole, Last.fm Ltd <jono@last.fm>                              *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -16,14 +17,14 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   51 Franklin Steet, Fifth Floor, Boston, MA  02111-1307, USA.          *
+ *   51 Franklin Steet, Fifth Floor, Boston, MA  02110-1301, USA.          *
  ***************************************************************************/
 
 #include "autoupdater.h"
 #include "logger.h"
 #include "exceptions.h"
-#include "winutils.h"
-#include "containerutils.h"
+#include "MooseCommon.h"
+#include "UnicornCommon.h"
 
 #ifdef WIN32
     #include <windows.h>
@@ -191,10 +192,10 @@ CAutoUpdater::downloadNext()
         emit updateDownloadDone(true, msg);
         return;
     }
-
+    
     QUrl url(current.GetDownloadURL());
 
-    mTransport = new Http(this);
+    mTransport = new CachedHttp(this);
     mTransport->setHost(url.host());
 
     connect(mTransport, SIGNAL(requestFinished(int, bool)),
@@ -204,7 +205,7 @@ CAutoUpdater::downloadNext()
     connect(mTransport, SIGNAL(stateChanged(int)),
             this,       SLOT  (statusChanged(int)));
 
-    mHttpId = mTransport->QHttp::get( url.path(), &mFile );
+    mHttpId = mTransport->get( url.path(), &mFile );
 
     mCancelled = false;
 }
@@ -222,6 +223,13 @@ CAutoUpdater::downloadDone(
 
     Q_ASSERT(mTransport);
 
+    QHttpResponseHeader header = mTransport->lastResponse();
+    int statusCode = header.statusCode();
+    
+    //Successfull response is 2xx
+    if (statusCode < 200 || statusCode >= 300 )
+        error = true;
+
     if (mCancelled)
     {
         LOG(3, "Download of update cancelled. Not emitting any signals.\n");
@@ -232,15 +240,17 @@ CAutoUpdater::downloadDone(
 
     if (error)
     {
+        QString errorString = header.reasonPhrase();
         // Something went wrong
         LOG(2, "Download of component failed. Error: " <<
-            mTransport->error() << " - " << mTransport->errorString() << "\n");
+            statusCode << " - " << errorString << "\n");
 
         mFile.close();
         mTransport->deleteLater();
 
-        emit updateDownloadDone(true, QString(tr("Download failed (%1)")).
-            arg(mTransport->errorString()));
+        emit updateDownloadDone(true, QString(tr("Download failed ( %1 - %2 )")).
+                                                    arg(statusCode).
+                                                    arg(errorString));
     }
     else
     {
@@ -305,19 +315,7 @@ CAutoUpdater::downloadDone(
                 return;
             }
 
-            // HACK! Turning into a bit of a hackfest this. Seeing as all the
-            // exes we use so far are Inno Setup ones, we can tack on
-            // a "/DIR" param with the correct plugin path (which we can't
-            // know at installer compile time) onto the end of the args string.
-            // This will most probably die horribly if run on a non-Inno installer.
-
-            // According to the Qt docs, to encode a " inside a string I should use 3
-            // quotes. However, if I do, Qt fucks it up and adds backslashes where
-            // there shouldn't be any so I'm not using "s. This means that the
-            // path is passed unquoted but Inno is clever enough to understand the
-            // whole path anyway. Thank God for Inno!
-
-			QString installDir = QDir::convertSeparators( current.GetInstallPath() );
+			QString installDir = QDir::toNativeSeparators( current.GetInstallPath() );
             QDir dir( installDir );
             if ( !dir.exists() )
             {
@@ -344,73 +342,49 @@ CAutoUpdater::downloadDone(
 					reinterpret_cast<const ushort*>( buf ) );
             }
 
+            installDir = "\"" + installDir + "\"";
+
+            // HACK! Turning into a bit of a hackfest this. Seeing as all the
+            // exes we use so far are Inno Setup ones, we can tack on
+            // a "/DIR" param with the correct plugin path (which we can't
+            // know at installer compile time) onto the end of the args string.
+            // This will most probably die horribly if run on a non-Inno installer.
             QString installArgs = current.GetInstallArgs() + " /DIR=" + installDir;
 
             LOG(3, "Running '" << installerPath << " " << installArgs << "'\n");
 
             // Run installer
-		    if ( ( QSysInfo::WindowsVersion & QSysInfo::WV_VISTA ) == 0 )
-			{
-				// Use QProcess if not on Vista to not break Win98
-				QProcess proc( this );
-				proc.setWorkingDirectory( QCoreApplication::applicationDirPath() );
-				installerPath += " " + installArgs;
-				proc.start( installerPath );
+			// Must use ShellExecute because otherwise the elevation dialog
+			// doesn't appear when launching the installer on Vista.
+            SHELLEXECUTEINFOW sei;
+            memset(&sei, 0, sizeof(sei));
 
-				if (!proc.waitForStarted())
-				{
-					LOG(1, "Couldn't start " << installerPath <<
-						". Error: " << proc.error() << "\n");
+            sei.cbSize = sizeof(sei);
+            sei.fMask  = SEE_MASK_NOCLOSEPROCESS | SEE_MASK_FLAG_DDEWAIT;
+            sei.hwnd   = GetForegroundWindow();
+            sei.lpVerb = L"open";
+            sei.lpFile = reinterpret_cast<LPCWSTR>(installerPath.utf16());
+            sei.lpParameters = reinterpret_cast<LPCWSTR>(installArgs.utf16());
+            sei.nShow  = SW_SHOWNORMAL;
 
-					QString msg = tr("Failed to install '%1'.").arg( installerPath );
-					emit updateDownloadDone(true, msg);
-					return;
-				}
-	            
-				if (!proc.waitForFinished( 10000 ))
-				{
-					LOG(1, installerPath << " never finished. " <<
-						 ". Error: " << proc.error() << "\n");
-
-					QString msg = tr("Failed to install '") + installerPath + tr("'.");
-					emit updateDownloadDone(true, msg);
-					return;
-				}
-			}
-			else
-			{
-				// On Vista, use ShellExecute because otherwise the elevation dialog
-				// doesn't appear when launching the installer
-                SHELLEXECUTEINFOW sei;
-                memset(&sei, 0, sizeof(sei));
-
-                sei.cbSize = sizeof(sei);
-                sei.fMask  = SEE_MASK_NOCLOSEPROCESS | SEE_MASK_FLAG_DDEWAIT;
-                sei.hwnd   = GetForegroundWindow();
-                sei.lpVerb = L"open";
-                sei.lpFile = reinterpret_cast<LPCWSTR>(installerPath.utf16());
-                sei.lpParameters = reinterpret_cast<LPCWSTR>(installArgs.utf16());
-                sei.nShow  = SW_SHOWNORMAL;
-
-                BOOL bOK = ShellExecuteExW(&sei);
-                if (bOK)
-                {
-                    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
-                    WaitForSingleObject(sei.hProcess, 5000);
-                    CloseHandle(sei.hProcess);
-                    QApplication::restoreOverrideCursor();
-                }
-                else
-                {
-                    LOG(1, "Couldn't ShellExecuteEx " << installerPath << " " <<
-                        installArgs << ". GetLastError: " << GetLastError() << "\n");
-
-                    QString msg = tr("Failed to install '") + installerPath + tr("'.");
-                    emit updateDownloadDone(true, msg);
-                    return;
-                }
-			
+            BOOL bOK = ShellExecuteExW(&sei);
+            if (bOK)
+            {
+                QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
+                WaitForSingleObject(sei.hProcess, 5000);
+                CloseHandle(sei.hProcess);
+                QApplication::restoreOverrideCursor();
             }
+            else
+            {
+                LOG(1, "Couldn't ShellExecuteEx " << installerPath << " " <<
+                    installArgs << ". GetLastError: " << GetLastError() << "\n");
+
+                QString msg = tr("Failed to install '") + installerPath + tr("'.");
+                emit updateDownloadDone(true, msg);
+                return;
+            }
+		
             
             if (mnCurrentDownload < (mnDownloadTasks - 1))
             {
@@ -475,10 +449,15 @@ CAutoUpdater::LaunchAppInstaller()
 	#endif
 
 	// Synchronous
-    QString tempUpdaterPath = savePath( "UpTemp.exe" );
+    QString tempUpdaterPath = MooseUtils::savePath( "Updaterd.exe" );
     QFile::remove( tempUpdaterPath );
+    if ( QFile::exists( tempUpdaterPath ) )
+    {
+        LOG(1, "Removal of old Updater.exe failed. System error code: " <<
+            GetLastError() << "\n");
+    }
+    
     bool ok = QFile::copy( updaterPath, tempUpdaterPath );
-
     if (!ok)
     {
         LOG(1, "Copying of Updater.exe failed. System error code: " <<
@@ -489,7 +468,7 @@ CAutoUpdater::LaunchAppInstaller()
     QStringList args;
     args << "R" << QString::fromStdString(sMutex) << msAppDLPath;
 
-    LOG(3, "Will execute: UpTemp.exe " << args.join(" ") << "\n");
+    LOG(3, "Will execute: " << tempUpdaterPath << args.join(" ") << "\n");
 
     QProcess proc( this );
     proc.setWorkingDirectory( QCoreApplication::applicationDirPath() );
@@ -510,6 +489,10 @@ CAutoUpdater::LaunchAppInstaller()
 
 	return true;
 
+#elif defined Q_WS_MAC
+	QProcess::startDetached( "/bin/bash", QStringList() << QApplication::applicationDirPath() + "/updater-autorestart.sh" << MooseUtils::savePath( "lastfm.pid" ) );
+		
+	return true;
 #else
     return false;
 #endif // WIN32
@@ -573,7 +556,7 @@ void
 CAutoUpdater::statusChanged(
     int state)
 {
-    QString msg = QHttpStateToString(state);
+    QString msg = UnicornUtils::QHttpStateToString(state);
     emit statusChange(msg);
 }
 
